import React, { useEffect, useState, ChangeEvent } from "react";
import { useNavigate, useOutletContext } from "react-router-dom";
import { getStatusColor } from "../common/lib/util";
import { JobPayloadType, ApplicationPayloadType } from "../common/types";
import Layout from "../Layouts/Layout";

const Applications = () => {
  const navigate = useNavigate();
  const job = useOutletContext<JobPayloadType>();
  const [applications, setApplications] = useState<ApplicationPayloadType[]>(
    []
  );

  useEffect(() => {
    if (job) {
      sortApplications("skills", job);
    }
  }, [job]);

  const onSelectSort = (e: ChangeEvent<HTMLSelectElement>) => {
    sortApplications(e.target.value, job);
  };
  const sortApplications = (sortBy: string, _job?: JobPayloadType | null) => {
    if (_job?.applications) {
      const _applications = _job.applications as ApplicationPayloadType[];
      if (sortBy === "skills") {
        _applications.sort((current, prev) => {
          const skill = current.score.primary + current.score.secondary;
          const prevSkill = prev.score.primary + prev.score.secondary;
          if (skill < prevSkill) {
            return 1;
          } else if (skill > prevSkill) {
            return -1;
          }
          return 0;
        });
      } else if (sortBy === "similarity") {
        _applications.sort((current, prev) => {
          if (current.score.similarity < prev.score.similarity) {
            return 1;
          } else if (current.score.similarity > prev.score.similarity) {
            return -1;
          }
          return 0;
        });
      } else {
        _applications.sort((current, prev) => {
          if (current.score.total < prev.score.total) {
            return 1;
          } else if (current.score.total > prev.score.total) {
            return -1;
          }
          return 0;
        });
      }

      setApplications([..._applications]);
    }
  };

  const renderApplications = (_application: any, index: number) => {
    const application = _application as ApplicationPayloadType;
    const skillsScore = `PS ${application.score.primary} + SS ${application.score?.secondary} `;
    const status = `badge ${getStatusColor(application.status)}`;
    const onClick = () =>
      navigate(`/applicant?jobId=${job._id}&applicationId=${_application._id}`);

    return (
      <tr key={index} onClick={onClick} style={{ cursor: "pointer" }}>
        <th scope="row">{application.candidate.name}</th>
        <td>{skillsScore}</td>
        <td>{application.score.similarity}</td>
        <td>{application.score.total}</td>
        <td>
          <span className={status}>{application.status}</span>
        </td>
        <td>{application.candidate.contacts?.phone || ""}</td>
      </tr>
    );
  };

  if (!job) return <Layout title="Applications"></Layout>;
  return (
    <>
      <h4>Applications</h4>
      <div className="mb-3 row">
        <label className="col-sm-2 col-form-label">Sort by</label>
        <div className="col-sm-4">
          <select
            className="form-select"
            aria-label="Default select example"
            onChange={onSelectSort}
          >
            <option value="skills">Skills score</option>
            <option value="similarity">Similarity</option>
            <option value="total">Total score</option>
          </select>
        </div>
      </div>

      <div className="card p-4 table-responsive">
        <table className="table table-hover table-sm">
          <thead>
            <tr>
              <th scope="col">Name</th>
              <th scope="col">Skills Score</th>
              <th scope="col">Similarity</th>
              <th scope="col">Total score</th>
              <th scope="col">Status</th>
              <th scope="col">Phone</th>
            </tr>
          </thead>
          <tbody>{applications?.map(renderApplications)}</tbody>
        </table>
      </div>
    </>
  );
};

export default Applications;
