import React, { useState, useMemo } from "react";
import { useDispatch, useSelector } from "react-redux";
import { applyForJob } from "../common/actions/user";
import { getStatusColor, getUserId } from "../common/lib/util";
import { JobPayloadType, JobType, Reducers, USER_TYPE } from "../common/types";
import Jobs from "../components/Jobs";
import Layout from "../Layouts/Layout";
import CreateUpdateJob from "../components/Modals/CreateUpdateJob";
import Profile from "../components/Profile";

const CandidateHome = () => {
  const dispatch = useDispatch();
  const [selectedJob, setSelectedJob] = useState<number>(0);
  const userId = useSelector(getUserId);
  const jobs = useSelector(
    (state: Reducers) => state.common.jobs
  ) as JobPayloadType[];
  const job = jobs.length > 0 ? jobs[selectedJob] : null;

  const renderSkills = (skill: string, index: number) => (
    <span className="skill display" key={index}>
      {skill}
    </span>
  );

  const alreadyApplied = useMemo(() => {
    const foundCandidate = jobs[selectedJob]?.applications?.findIndex(
      (_item) => _item.candidate === userId
    );
    if (foundCandidate !== undefined && foundCandidate >= 0) {
      const status = jobs[selectedJob]?.applications?.[foundCandidate].status;
      return {
        applied: true,
        status,
        color: getStatusColor(status),
      };
    } else {
      return { applied: false, status: "" };
    }
  }, [jobs, selectedJob, userId]);

  const onApply = () => {
    dispatch(applyForJob(jobs[selectedJob]._id || ""));
  };

  return (
    <div className="row">
      <div className="col-3 ">
        <Profile />
      </div>
      <div className="col">
        <Jobs
          jobs={jobs}
          onSelectJob={setSelectedJob}
          selectedIndex={selectedJob}
        />
      </div>
      <div className="col">
        {job && (
          <div className="card p-4 job-preview">
            <h5>{job.title}</h5>
            <label>
              <a href={`/organizations?id=${job.organization[0]._id}`}>
                By {job.organization[0].name}
              </a>
            </label>
            <p>
              Salary : {job.salary.min} - {job.salary.max} {job.salary.currency}{" "}
            </p>
            <p className="desc">{job.description}</p>
            <label>Primary skills</label>
            <div className="skills">{job.primarySkills.map(renderSkills)}</div>
            <label>Secondary skills</label>
            <div className="skills">
              {job.secondarySkills.map(renderSkills)}
            </div>
            {alreadyApplied.applied ? (
              <p className="h5 mt-2">
                Status :{" "}
                <label className={alreadyApplied.color}>
                  {alreadyApplied.status}
                </label>
              </p>
            ) : (
              <button className="btn btn-success mt-3" onClick={onApply}>
                Apply
              </button>
            )}
          </div>
        )}
      </div>
    </div>
  );
};

const OrganizationHome = () => {
  const [selectedJob, setSelectedJob] = useState<number>(0);
  const jobs = useSelector((state: Reducers) => state.common.jobs) as JobType[];
  const job = jobs.length > 0 ? jobs[selectedJob] : null;

  const renderSkills = (skill: string, index: number) => (
    <span className="skill" key={index}>
      {skill}
    </span>
  );

  return (
    <div className="row">
      <div className="col-3 ">
        <Profile />
        <CreateUpdateJob mode="create" />
      </div>
      <div className="col">
        <Jobs
          jobs={jobs}
          onSelectJob={setSelectedJob}
          selectedIndex={selectedJob}
        />
      </div>
      <div className="col">
        {job && (
          <div className="card p-4 job-preview">
            <h5>{job.title}</h5>
            <p>
              Salary : {job.salary.min} - {job.salary.max} {job.salary.currency}{" "}
            </p>
            <p className="desc">{job.description}</p>
            <label>Primary skills</label>
            <div className="skills">{job.primarySkills.map(renderSkills)}</div>
            <label>Secondary skills</label>
            <div className="skills">
              {job.secondarySkills.map(renderSkills)}
            </div>
            <div className="mt-2">
              <label>Applicattions : </label>{" "}
              <a href={`/applications?jobId=${job._id}`}>
                ({job.applications?.length}) Candidates
              </a>
            </div>
            <CreateUpdateJob defaultJob={job} mode="update" />
          </div>
        )}
      </div>
    </div>
  );
};

const Home = () => {
  const userType = useSelector((state: Reducers) => state.auth.userType);
  return (
    <Layout title="Vacancies and Jobs">
      {userType === USER_TYPE.ORGANIZATION ? (
        <OrganizationHome />
      ) : (
        <CandidateHome />
      )}
    </Layout>
  );
};

export default Home;
