import React, { ChangeEvent, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import { updateKeystrokeSettings } from "../common/actions/auth";
import { DEFAULT_CONTROLS } from "../common/config";
import { ControlsType, Reducers } from "../common/types";
import ChangePassword from "../components/ChangePassword";
import Charts from "../components/Charts";
import Layout from "../Layouts/Layout";
import CandidateProfile from "../components/CandidateProfile";
import ResultTable from "../components/ResultTable";

const Settings = () => {
  const dispatch = useDispatch();
  const keystrokeResult = useSelector(
    (state: Reducers) => state.auth.keystrokeResult
  );
  const [controls, setControls] = useState(
    keystrokeResult?.controls || DEFAULT_CONTROLS
  );

  const onToggleStandard = () =>
    setControls({
      ...controls,
      standard: { ...controls.standard, use: !controls.standard.use },
    });

  const onToggleAdvance = () =>
    setControls({
      ...controls,
      fullStandard: {
        ...controls.fullStandard,
        use: !controls.fullStandard.use,
      },
    });

  const onChangeKeystroke = (e: ChangeEvent<HTMLInputElement>) => {
    const [level, key] = e.target.name.split(".");
    const attr = controls[level as keyof ControlsType];

    setControls({
      ...controls,
      [level]: {
        ...attr,
        [key]: Number(e.target.value),
      },
    });
  };

  const onClickUpdateControls = () => {
    dispatch(updateKeystrokeSettings(controls));
  };

  return (
    <Layout title="Profile and Settings">
      <div className="row">
        <div className="col-8">
          <CandidateProfile />
          <ChangePassword />
        </div>
        <div className="col-4">
          <div className="card p-4 keystrokes">
            <h5>Keystroke-dynamics Settings</h5>
            <div className="keystrokes-settings">
              <div className="form-check form-switch">
                <label className="form-check-label">Standard Settings</label>
                <input
                  className="form-check-input"
                  type="checkbox"
                  role="switch"
                  checked={controls.standard.use}
                  onChange={onToggleStandard}
                />
              </div>
              <label htmlFor="customRange2" className="form-label">
                SD Multiplier : {controls.standard.sd}
              </label>
              <input
                type="range"
                className="form-range"
                min="0"
                max="2.5"
                step="0.1"
                value={controls.standard.sd}
                name="standard.sd"
                onChange={onChangeKeystroke}
              />
              <label htmlFor="customRange2" className="form-label">
                Threshold : {controls.standard.threshold}
              </label>
              <input
                type="range"
                className="form-range"
                min="1"
                max="100"
                value={controls.standard.threshold}
                name="standard.threshold"
                onChange={onChangeKeystroke}
              />
            </div>

            <div className="keystrokes-settings">
              <div className="form-check form-switch">
                <label className="form-check-label">Advance Settings</label>
                <input
                  className="form-check-input"
                  type="checkbox"
                  role="switch"
                  checked={controls.fullStandard.use}
                  onChange={onToggleAdvance}
                />
              </div>
              <label htmlFor="customRange2" className="form-label">
                Threshold : {controls.fullStandard.threshold}
              </label>
              <input
                type="range"
                className="form-range"
                min="0"
                max="2"
                step="0.1"
                value={controls.fullStandard.threshold}
                name="fullStandard.threshold"
                onChange={onChangeKeystroke}
              />
            </div>

            <button
              type="button"
              className="btn btn-primary"
              onClick={onClickUpdateControls}
            >
              Update
            </button>

            <hr />
            <h5>Last Login</h5>
            <ResultTable data={keystrokeResult?.result} />
            <Charts
              title="Overall keystroke"
              real={keystrokeResult?.db.full}
              attempts={keystrokeResult?.attempt.full}
            />
          </div>
        </div>
      </div>
    </Layout>
  );
};

export default Settings;
